#define		VERBOSE
#define		DEBUG
#undef		DEBUG
#define		TEST_MULTI			/* short file test */
#include	"pdtar.h"
#include	"pdproto.h"
#define		MAXSEC	15			/* max sectors per track */
#define		MAXTRY	5			/* number of biosdisk tries to make */
static		int		nhead;		/* head to read */
static		int		track;		/* track to read */
static		int		sector;		/* sector to read */
static		int		nsects = 1;	/* sectors to read from one track */
static		int		spt = MAXSEC;	/* sectors per track */
static		int		clunk;			/* err status from biosdisk */
static		int		chunk;			/* work variable */
static		int		tries;			/* tries before quit */
static		int		slack;			/* blocks that can be written */
static		int		drek;
/**/
#ifdef	VERBOSE
struct	VERBOSE	{
	unsigned	verror;
	char		*vmesg;
	}	barf[] = {
	0x00, "success",
	0x01, "bad command",
	0x02, "address mark not found",
	0x04, "record not found",
	0x05, "reset failed",
	0x07, "drive parameter activity failed",
	0x09, "attempt to dma over 64 k boundary",
	0x0B, "bad track flag detected",
	0x10, "bad ecc on disk read",
	0x11, "ecc corrected data error",
	0x20, "controller has died",
	0x40, "seek operation failed",
	0x80, "attachment failed to respond",
	0xBB, "undefined error",
	0xFF, "sense operation failed",
	69,	  "FUBAR",
	}	;
char	*
get_barf(ec)
NOSIGN	ec;
{
	auto	short	ix;
	for(ix = 0; barf[ix].verror != FUBAR; ix++)	{
		if(barf[ix].verror == ec)	return(barf[ix].vmesg);
	}
	return("FUBAR");
}
#endif	/*VERBOSE*/
/**/
rawreset(file)
int		file;
{
	return(absio(BIOS_RESET,file,NULL,0,0));
}
absio(type,file,buffer,blocks,start)
int		type;		/* operation */
int		file;		/* fd */
char	*buffer;	/* buffer */
int		blocks;		/* blocks to read or write */
int		start;		/* starting block number */
{
	static	char	*here = "absio";
	auto	int		turkey = 0;
#ifdef	DEBUGR
	fprintf(stderr,"absio(%u,%u,%u,%u,%u)\n",
		type,file,buffer,blocks,start);
#endif
/**/
	drek = clunk = 0;
/**/
	switch(type)	{
		case	BIOS_RESET:
			biosdisk(type,0,0,0,0,0,NULL);
			return(0);
		case	BIOS_READ:
		case	BIOS_WRITE:
			while(clunk == 0 && blocks >= 1)	{
/*
fortunately Xenix uses a very simple scheme for writing blocks
(sectors) to disk.  no complext mapping is needed.  beginning with
track 0, head 0, nsectors are written, then track 0, head 1, etc.
write as many sectors as remain for the current track.  this method
eliminates the need for flushing.  for large files, this becomes
per track buffering.
*/
			sector = start % spt;
			chunk = start / spt;
			track = chunk / 2;
			nhead = chunk % 2;
			slack = spt - sector;
			if(slack > blocks)	slack = blocks;
			if(turkey)			slack = 1;
#ifdef	ONE_BLOCK
			slack = 1;
#endif	/*ONE_BLOCK*/
#ifdef	DEBUG
fprintf(stderr,"rdwrbios: H %1u T %2u S %2u L %2u B %6u\n",
	nhead,track,sector,slack,buffer);
#endif
			clunk = rdwrbios(type,file,nhead,track,sector,slack,buffer);
/**/
			if(clunk == 0x04 && turkey == 0)	{
				turkey = 1;
				clunk = 0;
#ifdef	VERBOSE
				fprintf(stderr,"RETRY: H %1u T %2u S %2u L %2u\n",
					nhead,track,sector,slack);
#endif
				continue;
				}
			drek += slack;
			start += slack;
			blocks -= slack;
			buffer += (slack * BLKSIZ);
			}
			break;
		default:
			where(here);
			fprintf(stderr,"<bad call> %d\n",type);
			break;
	}
/**/
	if(clunk)	{
		where(here);
#ifdef	VERBOSE
		fprintf(stderr,"<%s> %d %d\n",get_barf(clunk),type,file);
#else	/*VERBOSE*/
		fprintf(stderr,"<bios error> %d %x\n",clunk,clunk);
#endif	/*VERBOSE*/
		terror(FUBAR);
		}
	return(drek);
}
setspt(devsize)
int		devsize;	/* size of device in BLKSIZ units */
{
	static	char	*here = "setspt";
	switch(devsize)	{
		case	0:		device.bpr = device.bpw = spt = nsects = 0;break;
#ifdef	TEST_MULTI
		case	50:		device.bpr = device.bpw = spt = nsects = 9;break;
#endif
		case	720:	device.bpr = device.bpw = spt = nsects = 9;break;
		case	1440:	device.bpr = device.bpw = spt = nsects = 9;break;
		case	2400:	device.bpr = device.bpw = spt = nsects = 15;break;
		default:		where(here);
						fprintf(stderr,"<unknown size> '%d'\n",devsize);
						terror(FUBAR);
		}
	return(spt);
}
rdwrbios(type,file,nhead,track,sector,blocks,buffer)
int		type;
int		file;
int		nhead;
int		track;
int		sector;
int		blocks;
char	*buffer;
{
/*
|	try MAXTRY times to perform operation.  break loop if clunk == 0 for
|	success or too many tries.
*/
	for(clunk = 1 , tries = 0; clunk && tries < MAXTRY; tries++)	{
		clunk = biosdisk(type,file,nhead,track,sector + 1,blocks,buffer);
	}
	return(clunk);
}
setdrive(drive)
int		drive;
{
	static	char	*here = "setdrive";
	device.rawd = drive;
	if(islower(device.rawd))	device.rawd = toupper(device.rawd);
	if(device.rawd == 'C' || device.rawd == 'D')	{
		where(here);
		fprintf(stderr,"<invalid drive> '%c'\n",device.rawd);
		exit(FUBAR);
		}
	device.rawd -= 'A';
	rfunc = rawread;
	wfunc = rawwrite;
	return(device.rawd);
}
